@@ -104,6 +104,16 @@ class ParseUriTest(unittest.TestCase):
         self.assertEqual(parsed_uri.host, "hostname")
         self.assertEqual(parsed_uri.port, 1234)
 
+    def test_s3_handles_fragments(self):
+        uri_str = 's3://bucket-name/folder/picture #1.jpg'
+        parsed_uri = smart_open_lib._parse_uri(uri_str)
+        self.assertEqual(parsed_uri.key_id, "folder/picture #1.jpg")
+
+    def test_s3_handles_querystring(self):
+        uri_str = 's3://bucket-name/folder/picture1.jpg?bar'
+        parsed_uri = smart_open_lib._parse_uri(uri_str)
+        self.assertEqual(parsed_uri.key_id, "folder/picture1.jpg?bar")
+
     def test_s3_invalid_url_atmark_in_bucket_name(self):
         self.assertRaises(ValueError, smart_open_lib._parse_uri, "s3://access_id:access_secret@my@bucket@port/mykey")
 
@@ -256,7 +266,7 @@ class SmartOpenHttpTest(unittest.TestCase):
         smart_open_object = smart_open.smart_open(
             'http://127.0.0.1/data%s%s' % (suffix, '?some_param=some_val' if query else ''))
 
-        # decompress the xz and get the same md5 hash
+        # decompress the file and get the same md5 hash
         self.assertEqual(smart_open_object.read(), raw_data)
 
     @unittest.skipIf(six.PY2, 'gzip support for Py2 is not implemented yet')
@@ -268,10 +278,6 @@ class SmartOpenHttpTest(unittest.TestCase):
         """Can open bzip2 via http?"""
         self._test_compressed_http(".bz2", False)
 
-    def test_http_xz(self):
-        """Can open xz via http?"""
-        self._test_compressed_http(".xz", False)
-
     @unittest.skipIf(six.PY2, 'gzip support for Py2 is not implemented yet')
     def test_http_gz_query(self):
         """Can open gzip via http with a query appended to URI?"""
@@ -281,10 +287,6 @@ class SmartOpenHttpTest(unittest.TestCase):
         """Can open bzip2 via http with a query appended to URI?"""
         self._test_compressed_http(".bz2", True)
 
-    def test_http_xz_query(self):
-        """Can open xz via http with a query appended to URI?"""
-        self._test_compressed_http(".xz", True)
-
 
 def make_buffer(cls=six.BytesIO, initial_value=None, name=None):
     """
@@ -1014,20 +1016,6 @@ class CompressionFormatTest(unittest.TestCase):
         """Can write and read bz2?"""
         self.write_read_assertion('.bz2')
 
-    def test_write_read_xz(self):
-        """Can write and read xz2?"""
-        self.write_read_assertion('.xz')
-
-    def test_read_real_xz(self):
-        """Can read a real xz file."""
-        base_path = os.path.join(CURR_DIR, 'test_data/crime-and-punishment.txt')
-        head_path = os.path.join(CURR_DIR, 'test_data/crime-and-punishment.txt.xz')
-        with smart_open.smart_open(head_path) as f:
-            smart_data = f.read()
-        with open(base_path, 'rb') as f:
-            orig_data = f.read()
-        self.assertEqual(smart_data, orig_data)
-
 
 class MultistreamsBZ2Test(unittest.TestCase):
     """
